
RUBYNODE_H = 'rubynode.h'
NODEDUMP_C = 'ndcore.c'


require 'mkmf'


def main
  create_rubynode_h()
  if n = unknown_nodes()
    $stderr.puts "unknown node detected: #{n.join(' ')}"
    $stderr.puts 'abort.'
    exit 1
  end
  create_makefile 'ndcore'
end


def create_rubynode_h
  File.open(node_h_path()) {|r|
      while line = r.gets
        break if /enum node_type/ === line
      end
      File.open('rubynode.h', 'w') {|f|
          f.puts 'const char *node_names[] = {'
          while line = r.gets
            break if /\};/ === line
            case line
            when /\A\#/
              f.print line
            when /\A\s*(\w+)(,)?/
              name = $1
              comma = $2
              f.puts %Q<\t"#{name}"#{comma}>
              f.puts "#define #{name} #{name}"
            end
          end
          f.puts '};'
      }
  }
end

def node_h_path
  header_dir() + '/node.h'
end

def header_dir
  ruby_header_dir('$libdir/ruby/$MAJOR.$MINOR/$arch') or
  ruby_header_dir('$srcdir') or
  ($stderr.puts 'archdir not exist: abort.'; exit 1)
end

require 'rbconfig'

def ruby_header_dir( fmt )
  dir = expand_params(fmt, Config::CONFIG)
  if File.exist? dir + '/ruby.h'
    dir
  else
    nil
  end
end

def expand_params( fmt, params )
  fmt.gsub(/\$(\w+)/) { params[$1] }
end


DUMMY_NODES = %w(
    NODE_MEMO
    NODE_LAST
    NODE_ALLOCA
)
    # NODE_CNAME   # unused in 1.7
    # NODE_CFUNC   # now he have NodeDump#dump.
    # NODE_IFUNC   # now he have NodeDump#dump.

def unknown_nodes
  unknown = defined_nodes() - known_nodes() - DUMMY_NODES
  if unknown.empty?
    nil
  else
    unknown
  end
end

def defined_nodes
  File.open(RUBYNODE_H) {|f|
      return f.map {|line| line.slice(/#define (\w+)/, 1) }.compact
  }
end

def known_nodes
  File.open(File.join(File.dirname($0), NODEDUMP_C)) {|f|
      return f.grep(/case\s+(NODE_\w+):/) { $1 }
  }
end


main
